package com.sun.jmx.remote.security;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.ListenerNotFoundException;
import java.util.Set;
import javax.management.QueryExp;
import javax.management.IntrospectionException;
import javax.management.MBeanInfo;
import javax.management.loading.ClassLoaderRepository;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.OperationsException;
import java.io.ObjectInputStream;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.ReflectionException;
import javax.management.ObjectInstance;
import javax.management.InstanceNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.remote.MBeanServerForwarder;

public abstract class MBeanServerAccessController implements MBeanServerForwarder
{
    private MBeanServer mbs;
    
    public MBeanServerAccessController() {
    }
    
    @Override
    public MBeanServer getMBeanServer() {
        return this.mbs;
    }
    
    @Override
    public void setMBeanServer(MBeanServer mbs) {
        if (mbs == null) {
            throw new IllegalArgumentException("Null MBeanServer");
        }
        if (this.mbs != null) {
            throw new IllegalArgumentException("MBeanServer object already initialized");
        }
        this.mbs = mbs;
        return;
    }
    
    protected abstract void checkRead();
    
    protected abstract void checkWrite();
    
    protected void checkCreate(String s) {
        this.checkWrite();
    }
    
    protected void checkUnregister(ObjectName objectName) {
        this.checkWrite();
    }
    
    @Override
    public void addNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        this.checkRead();
        this.getMBeanServer().addNotificationListener(objectName, notificationListener, notificationFilter, o);
    }
    
    @Override
    public void addNotificationListener(ObjectName objectName, ObjectName objectName2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        this.checkRead();
        this.getMBeanServer().addNotificationListener(objectName, objectName2, notificationFilter, o);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        this.checkCreate(s);
        if (System.getSecurityManager() == null) {
            Object instantiate = this.getMBeanServer().instantiate(s);
            this.checkClassLoader(instantiate);
            return this.getMBeanServer().registerMBean(instantiate, objectName);
        }
        return this.getMBeanServer().createMBean(s, objectName);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        this.checkCreate(s);
        if (System.getSecurityManager() == null) {
            Object instantiate = this.getMBeanServer().instantiate(s, array, array2);
            this.checkClassLoader(instantiate);
            return this.getMBeanServer().registerMBean(instantiate, objectName);
        }
        return this.getMBeanServer().createMBean(s, objectName, array, array2);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        this.checkCreate(s);
        if (System.getSecurityManager() == null) {
            Object instantiate = this.getMBeanServer().instantiate(s, objectName2);
            this.checkClassLoader(instantiate);
            return this.getMBeanServer().registerMBean(instantiate, objectName);
        }
        return this.getMBeanServer().createMBean(s, objectName, objectName2);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        this.checkCreate(s);
        if (System.getSecurityManager() == null) {
            Object instantiate = this.getMBeanServer().instantiate(s, objectName2, array, array2);
            this.checkClassLoader(instantiate);
            return this.getMBeanServer().registerMBean(instantiate, objectName);
        }
        return this.getMBeanServer().createMBean(s, objectName, objectName2, array, array2);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(ObjectName name, byte[] data) throws InstanceNotFoundException, OperationsException {
        this.checkRead();
        return this.getMBeanServer().deserialize(name, data);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(String className, byte[] data) throws OperationsException, ReflectionException {
        this.checkRead();
        return this.getMBeanServer().deserialize(className, data);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(String className, ObjectName loaderName, byte[] data) throws InstanceNotFoundException, OperationsException, ReflectionException {
        this.checkRead();
        return this.getMBeanServer().deserialize(className, loaderName, data);
    }
    
    @Override
    public Object getAttribute(ObjectName objectName, String s) throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException {
        this.checkRead();
        return this.getMBeanServer().getAttribute(objectName, s);
    }
    
    @Override
    public AttributeList getAttributes(ObjectName objectName, String[] array) throws InstanceNotFoundException, ReflectionException {
        this.checkRead();
        return this.getMBeanServer().getAttributes(objectName, array);
    }
    
    @Override
    public ClassLoader getClassLoader(ObjectName objectName) throws InstanceNotFoundException {
        this.checkRead();
        return this.getMBeanServer().getClassLoader(objectName);
    }
    
    @Override
    public ClassLoader getClassLoaderFor(ObjectName objectName) throws InstanceNotFoundException {
        this.checkRead();
        return this.getMBeanServer().getClassLoaderFor(objectName);
    }
    
    @Override
    public ClassLoaderRepository getClassLoaderRepository() {
        this.checkRead();
        return this.getMBeanServer().getClassLoaderRepository();
    }
    
    @Override
    public String getDefaultDomain() {
        this.checkRead();
        return this.getMBeanServer().getDefaultDomain();
    }
    
    @Override
    public String[] getDomains() {
        this.checkRead();
        return this.getMBeanServer().getDomains();
    }
    
    @Override
    public Integer getMBeanCount() {
        this.checkRead();
        return this.getMBeanServer().getMBeanCount();
    }
    
    @Override
    public MBeanInfo getMBeanInfo(ObjectName objectName) throws InstanceNotFoundException, IntrospectionException, ReflectionException {
        this.checkRead();
        return this.getMBeanServer().getMBeanInfo(objectName);
    }
    
    @Override
    public ObjectInstance getObjectInstance(ObjectName objectName) throws InstanceNotFoundException {
        this.checkRead();
        return this.getMBeanServer().getObjectInstance(objectName);
    }
    
    @Override
    public Object instantiate(String s) throws ReflectionException, MBeanException {
        this.checkCreate(s);
        return this.getMBeanServer().instantiate(s);
    }
    
    @Override
    public Object instantiate(String s, Object[] array, String[] array2) throws ReflectionException, MBeanException {
        this.checkCreate(s);
        return this.getMBeanServer().instantiate(s, array, array2);
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName) throws ReflectionException, MBeanException, InstanceNotFoundException {
        this.checkCreate(s);
        return this.getMBeanServer().instantiate(s, objectName);
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, MBeanException, InstanceNotFoundException {
        this.checkCreate(s);
        return this.getMBeanServer().instantiate(s, objectName, array, array2);
    }
    
    @Override
    public Object invoke(ObjectName objectName, String s, Object[] array, String[] array2) throws InstanceNotFoundException, MBeanException, ReflectionException {
        this.checkWrite();
        this.checkMLetMethods(objectName, s);
        return this.getMBeanServer().invoke(objectName, s, array, array2);
    }
    
    @Override
    public boolean isInstanceOf(ObjectName objectName, String s) throws InstanceNotFoundException {
        this.checkRead();
        return this.getMBeanServer().isInstanceOf(objectName, s);
    }
    
    @Override
    public boolean isRegistered(ObjectName objectName) {
        this.checkRead();
        return this.getMBeanServer().isRegistered(objectName);
    }
    
    @Override
    public Set<ObjectInstance> queryMBeans(ObjectName objectName, QueryExp queryExp) {
        this.checkRead();
        return this.getMBeanServer().queryMBeans(objectName, queryExp);
    }
    
    @Override
    public Set<ObjectName> queryNames(ObjectName objectName, QueryExp queryExp) {
        this.checkRead();
        return this.getMBeanServer().queryNames(objectName, queryExp);
    }
    
    @Override
    public ObjectInstance registerMBean(Object o, ObjectName objectName) throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
        this.checkWrite();
        return this.getMBeanServer().registerMBean(o, objectName);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener) throws InstanceNotFoundException, ListenerNotFoundException {
        this.checkRead();
        this.getMBeanServer().removeNotificationListener(objectName, notificationListener);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        this.checkRead();
        this.getMBeanServer().removeNotificationListener(objectName, notificationListener, notificationFilter, o);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, ObjectName objectName2) throws InstanceNotFoundException, ListenerNotFoundException {
        this.checkRead();
        this.getMBeanServer().removeNotificationListener(objectName, objectName2);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, ObjectName objectName2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        this.checkRead();
        this.getMBeanServer().removeNotificationListener(objectName, objectName2, notificationFilter, o);
    }
    
    @Override
    public void setAttribute(ObjectName objectName, Attribute attribute) throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        this.checkWrite();
        this.getMBeanServer().setAttribute(objectName, attribute);
    }
    
    @Override
    public AttributeList setAttributes(ObjectName objectName, AttributeList list) throws InstanceNotFoundException, ReflectionException {
        this.checkWrite();
        return this.getMBeanServer().setAttributes(objectName, list);
    }
    
    @Override
    public void unregisterMBean(ObjectName objectName) throws InstanceNotFoundException, MBeanRegistrationException {
        this.checkUnregister(objectName);
        this.getMBeanServer().unregisterMBean(objectName);
    }
    
    private void checkClassLoader(Object o) {
        if ((o instanceof ClassLoader)) {
            throw new SecurityException("Access denied! Creating an MBean that is a ClassLoader is forbidden unless a security manager is installed.");
        }
        return;
    }
    
    private void checkMLetMethods(ObjectName objectName, String s) throws InstanceNotFoundException {
        if (System.getSecurityManager() != null) {
            return;
        }
        if (!s.equals("addURL") && !s.equals("getMBeansFromURL")) {
            return;
        }
        if (!this.getMBeanServer().isInstanceOf(objectName, "javax.management.loading.MLet")) {
            return;
        }
        if (s.equals("addURL")) {
            throw new SecurityException("Access denied! MLet method addURL cannot be invoked unless a security manager is installed.");
        }
        if (!"true".equalsIgnoreCase((String)AccessController.doPrivileged(new GetPropertyAction("jmx.remote.x.mlet.allow.getMBeansFromURL")))) {
            throw new SecurityException("Access denied! MLet method getMBeansFromURL cannot be invoked unless a security manager is installed or the system property -Djmx.remote.x.mlet.allow.getMBeansFromURL=true is specified.");
        }
        return;
    }
}
